use bcup::{models::ServerConfiguration, services::BackupServerService};
use futures::StreamExt;
use surrealdb::{engine::remote::ws::Ws, Notification, Result, Surreal};

#[tokio::main]
async fn main() -> surrealdb::Result<()> {
    let db = Surreal::new::<Ws>("127.0.0.1:8000").await?;

    db.use_ns("test").use_db("test").await?;

    let service = BackupServerService::new(db.clone());

    println!("Prepared to listen");
    let mut stream = db.select("rest_server").live().await?;
    while let Some(result) = stream.next().await {
        handle(result, &service).await;
    }

    Ok(())
}
async fn handle(result: Result<Notification<ServerConfiguration>>, service: &BackupServerService) {
    println!("Something to handle");
    match result {
        Ok(notification) => {
            let conf = &notification.data;
            if conf.state.is_pending() {
                process_new_item(conf, service).await;
            }
        }
        Err(error) => eprintln!("{error}"),
    }
}

async fn process_new_item(conf: &ServerConfiguration, service: &BackupServerService) {
    println!("The following item has to be processed {:?}", conf);

    service.spawn_restic_container(conf.directory.clone()).await;
    service
        .add_proxy_configuration(conf.directory.clone())
        .await;
    service.mark_data_as_processed(conf).await;
}
